@@ -2,7 +2,7 @@
 
 /**
   * SquirrelMail Address Book Grouping and Pagination Plugin
-  * Copyright (c) 2008 Paul Lesniewski <paul@squirrelmail.org>,
+  * Copyright (c) 2008-2009 Paul Lesniewski <paul@squirrelmail.org>
   * Licensed under the GNU GPL. For full terms see the file COPYING.
   *
   * @package plugins
@@ -45,7 +45,7 @@ function abook_group_pagination_init()
 
    if (!@include_once (SM_PATH . 'config/config_abook_group_pagination.php'))
       if (!@include_once (SM_PATH . 'plugins/abook_group_pagination/config.php'))
-         if (!@include_once (SM_PATH . 'plugins/abook_group_pagination/config.sample.php'))
+         if (!@include_once (SM_PATH . 'plugins/abook_group_pagination/config_default.php'))
             return FALSE;
 
    return TRUE;
@@ -54,7 +54,7 @@ function abook_group_pagination_init()
    return load_config('abook_group_pagination',
                       array('../../config/config_abook_group_pagination.php',
                             'config.php',
-                            'config.sample.php'),
+                            'config_default.php'),
                       TRUE, TRUE);
 ----------- */
 
@@ -69,7 +69,7 @@ function abook_group_pagination_init()
 function abook_group_pagination_display_options_do()
 {
 
-   global $data_dir, $username, $agp_abook_pagination,
+   global $data_dir, $username, $agp_abook_pagination, $agp_enable_show_all,
           $agp_abook_pagination_allow_override, $agp_grouping,
           $optpage_data;
 
@@ -95,6 +95,17 @@ function abook_group_pagination_display_options_do()
          'posvals'       => array(0 => _("No Pagination"), 5 => 5, 10 => 10, 15 => 15, 20 => 20, 25 => 25, 30 => 30, 40 => 40, 50 => 50, 60 => 60, 70 => 70, 80 => 80, 90 => 90, 100 => 100, 125 => 125, 150 => 150, 175 => 175, 200 => 200, 250 => 250, 300 => 300, 400 => 400, 500 => 500),
          'refresh'       => SMOPT_REFRESH_NONE,
       );
+
+      $agp_enable_show_all = getPref($data_dir, $username,
+                                     'agp_enable_show_all',
+                                     $agp_enable_show_all);
+      $my_optpage_values[] = array(
+         'name'          => 'agp_enable_show_all',
+         'caption'       => _("Enable \"Show All\" Link"),
+         'type'          => SMOPT_TYPE_BOOLEAN,
+         'initial_value' => $agp_enable_show_all,
+         'refresh'       => SMOPT_REFRESH_NONE,
+      );
    }
 
 
@@ -115,6 +126,17 @@ function abook_group_pagination_display_options_do()
          'refresh'       => SMOPT_REFRESH_NONE,
          'save'          => 'agp_save_abook_groups',
       );
+
+
+      $agp_group_select_style = getPref($data_dir, $username, 'agp_group_select_style', 'multiple');
+      $my_optpage_values[] = array(
+         'name'          => 'agp_group_select_style',
+         'caption'       => _("Group Selection Style"),
+         'type'          => SMOPT_TYPE_STRLIST_RADIO,
+         'posvals'       => array('multiple' => _("Multiple"), 'single' => _("Single")),
+         'initial_value' => $agp_group_select_style,
+         'refresh'       => SMOPT_REFRESH_NONE,
+      );
    }
 
 
@@ -130,11 +152,15 @@ function abook_group_pagination_display_options_do()
 
 
       // 1.4.x - add an address book section to the display options
+      //         (be careful to merge with other options from other
+      //         plugins)
       //
       else
       {  
-         $optpage_data['grps']['abook_group_pagination'] = _("Address Book");
-         $optpage_data['vals']['abook_group_pagination'] = $my_optpage_values;
+         $optpage_data['grps']['address_book'] = _("Address Book");
+         if (empty($optpage_data['vals']['address_book']))
+            $optpage_data['vals']['address_book'] = array();
+         $optpage_data['vals']['address_book'] = array_merge($optpage_data['vals']['address_book'], $my_optpage_values);
       }  
 
    }  
@@ -290,11 +316,13 @@ function get_abook_paginator_link($backend_number, $page_number, $text, $page_si
   *                           which includes information about its
   *                           parent backend and especially any pagination
   *                           information needed herein
+  * @param boolean $enable_show_all_link When TRUE, a "Show All" link will
+  *                                      be added (OPTIONAL; default is TRUE).
   *
   * @return string The pagination string
   *
   */
-function get_abook_paginator_string($abook_entry)
+function get_abook_paginator_string($abook_entry, $enable_show_all_link=TRUE)
 {
 
    if (empty($abook_entry['page_number'])
@@ -303,6 +331,13 @@ function get_abook_paginator_string($abook_entry)
       return '';
 
 
+   // when we are showing all, we merely need to generate a "Paginate" link
+   // ("Paginate" is in SquirrelMail text domain!)
+   //
+   if ($enable_show_all_link && $abook_entry['page_number'] == 'all')
+      return get_abook_paginator_link($abook_entry['backend'], 1, _("Paginate")) . '&nbsp;';
+
+
    global $max_pagination_links_to_show, $small_paginator_text;
    if ($max_pagination_links_to_show < 3) $max_pagination_links_to_show = 3;
 
@@ -379,9 +414,15 @@ function get_abook_paginator_string($abook_entry)
    if ($show_elipses_after)
       $pagination_str .= '...' . $nbsp
                       . get_abook_paginator_link($backend_number, $total_pages, $total_pages, $page_size) . $nbsp;
-    if ($small_paginator_text) $pagination_str .= '</small>';
 
-    return $pagination_str;
+
+   // "Show All" comes from SquirrelMail text domain!
+   //
+   if ($enable_show_all_link) $pagination_str .= '|' . $nbsp . get_abook_paginator_link($backend_number, 'all', _("Show All")) . $nbsp;
+
+   if ($small_paginator_text) $pagination_str .= '</small>';
+
+   return $pagination_str;
 
 }
 
@@ -403,11 +444,11 @@ function get_abook_group_selector_string($backend)
 {
 
    global $data_dir, $username, $agp_grouping,
-          $agp_group_selections, $agp_abook_backends,
-          $agp_pagination;
+          $agp_group_selections;
 //TODO: note that we could actually put $agp_unrelated_get_args into a hidden form field, but this should not be necessary (really, even saving them when making GET links is overkill)
 
    abook_group_pagination_init();
+   $ret = '';
 
    $agp_groups = array();
    if ($agp_grouping)
@@ -420,34 +461,6 @@ function get_abook_group_selector_string($backend)
       return '';
 
 
-   // save the page number(s) if we can find them (only needed in 1.4.x)
-   //
-   // note that we only need to push these to the form once,
-   // so we keep track of whether or not we've done that yet
-   //
-   $ret = '';
-   if (!check_sm_version(1, 5, 2))
-   {
-      static $agp_save_pagination_link_uris = FALSE;
-      if (!$agp_save_pagination_link_uris)
-      {
-         $agp_save_pagination_link_uris = TRUE;
-
-         foreach ($agp_abook_backends as $backend)
-         {
-            if (!empty($agp_pagination[$backend]['page_number']))
-               $ret .= '<input type="hidden" name="backend_page_number_'
-                    . $backend . '" value="' . $agp_pagination[$backend]['page_number']
-                    . '" />';
-            if (!empty($agp_pagination[$backend]['page_size']))
-               $ret .= '<input type="hidden" name="backend_page_size_'
-                    . $backend . '" value="' . $agp_pagination[$backend]['page_size']
-                    . '" />';
-         }
-      }
-   }
-
-
    // grab groups that are already selected, if any
    //
    if (!empty($agp_group_selections[$backend])
@@ -460,6 +473,9 @@ function get_abook_group_selector_string($backend)
    // build the selector
    //
    sq_change_text_domain('abook_group_pagination');
+   $agp_group_select_style = getPref($data_dir, $username, 'agp_group_select_style', 'multiple');
+   if ($agp_group_select_style != 'multiple')
+      array_unshift($agp_groups, _("-- None --"));
    //$size = min(3, sizeof($agp_groups));
    $size = 4;
 
@@ -473,6 +489,7 @@ function get_abook_group_selector_string($backend)
       $oTemplate->assign('backend', $backend);
       $oTemplate->assign('all_groups', $agp_groups);
       $oTemplate->assign('selected_groups', $groups);
+      $oTemplate->assign('select_style', $agp_group_select_style);
       $ret .= $oTemplate->fetch('plugins/abook_group_pagination/group_selector.tpl');
    }
 
@@ -481,22 +498,31 @@ function get_abook_group_selector_string($backend)
    //
    else
    {
-      $ret .= '<table width="1%" cellpadding="0" cellspacing="2"><tr><td>'
-           . '<select name="abook_groups_' . $backend . '[]" multiple="multiple" size="' . $size . '">';
+      if ($agp_group_select_style == 'multiple')
+         $ret .= '<table width="1%" cellpadding="0" cellspacing="2"><tr><td>';
+
+      $ret .= '<select name="abook_groups_' . $backend . '[]"';
+      if ($agp_group_select_style == 'multiple')
+         $ret .= ' multiple="multiple" size="' . $size . '"';
+      $ret .= '>';
       foreach ($agp_groups as $group)
       {
          $ret .= '<option value="' . $group . '"'
-              . (in_array($group, $groups) ? ' selected="selected"' : '')
-              . '>' . $group . "</option>\n";
+               . (in_array($group, $groups) ? ' selected="selected"' : '')
+               . '>' . $group . "</option>\n";
       }
-      $ret .= '</select></td><td valign="bottom">'
+      $ret .= '</select>'
+           . ($agp_group_select_style == 'multiple' ? '</td><td valign="bottom">' : ' ')
            . '<input type="submit" name="abook_groupdel_' . $backend . '" value="'
-           . _("Remove") . '" /><br />'
+           . _("Remove") . '" />'
+           . ($agp_group_select_style == 'multiple' ? '<br />' : ' ')
            . '<input type="submit" name="abook_groupadd_' . $backend . '" value="'
-           . _("Add") . '" /><br />'
+           . _("Add") . '" />'
+           . ($agp_group_select_style == 'multiple' ? '<br />' : ' ')
            . '<input type="submit" name="abook_group_filter_'
            . $backend . '" value="' . _("Go")
-           . '" /></td></tr></table>';
+           . '" />'
+           . ($agp_group_select_style == 'multiple' ? '</td></tr></table>' : '');
    }
 
 
@@ -535,10 +561,44 @@ function add_pagination_and_grouping_do($abook_entry)
    }
 
 
-   global $color;
+   global $color, $agp_abook_pagination_allow_override, $agp_enable_show_all,
+          $username, $data_dir, $agp_pagination, $agp_abook_backends;
    $nbsp = '&nbsp;';
 
-   $paginator = get_abook_paginator_string($abook_entry);
+
+
+   abook_group_pagination_init();
+   if (/*Already done above: !check_sm_version(1, 5, 2) && */
+       $agp_abook_pagination_allow_override)
+      $agp_enable_show_all = getPref($data_dir, $username, 'agp_enable_show_all', $agp_enable_show_all);
+   $paginator = get_abook_paginator_string($abook_entry, $agp_enable_show_all);
+
+
+   // save the page number(s) if we can find them and select
+   // all info (only needed in 1.4.x)
+   //
+   // note that we only need these hidden inputs once on the page,
+   // so use a static flag to eliminate duplicate output
+   //
+   static $hidden_inputs_are_built = FALSE;
+   $hidden_inputs = '';
+   if (!empty($paginator) && !$hidden_inputs_are_built)
+   {
+      $hidden_inputs_are_built = TRUE;
+
+      foreach ($agp_abook_backends as $backend)
+      {
+         if (!empty($agp_pagination[$backend]['page_number']))
+            $hidden_inputs .= '<input type="hidden" name="backend_page_number_'
+                            . $backend . '" value="' . $agp_pagination[$backend]['page_number']
+                            . '" />';
+         if (!empty($agp_pagination[$backend]['page_size']))
+            $hidden_inputs .= '<input type="hidden" name="backend_page_size_'
+                            . $backend . '" value="' . $agp_pagination[$backend]['page_size']
+                            . '" />';
+      }
+   }
+
 
    // play nice with the Select Range plugin
    //
@@ -553,7 +613,9 @@ function add_pagination_and_grouping_do($abook_entry)
    }
 
    if (!empty($paginator))
-      $pagination_string = html_tag('td', $paginator, 'left', $color[0], 'width="50%"');
+      $pagination_string = html_tag('td', $paginator . $hidden_inputs, 'left', $color[0], 'width="50%"');
+
+
 
    $group_selector = get_abook_group_selector_string($abook_entry['backend']);
    if (!empty($group_selector))
@@ -614,7 +676,7 @@ function filter_address_book_listing_do(&$args)
           $agp_abook_pagination_allow_override, $agp_grouping,
           $agp_abook_backends, $agp_group_selections,
           $agp_pagination, $agp_backends_to_filter,
-          $agp_backend_sources;
+          $agp_backend_sources, $agp_enable_show_all;
 
    abook_group_pagination_init();
 
@@ -626,9 +688,14 @@ function filter_address_book_listing_do(&$args)
    if (check_sm_version(1, 5, 2))
       $agp_abook_pagination = 0;
    else if ($agp_abook_pagination_allow_override)
+   {
       $agp_abook_pagination = getPref($data_dir, $username,
                                      'agp_abook_pagination',
                                      $agp_abook_pagination);
+      $agp_enable_show_all = getPref($data_dir, $username,
+                                     'agp_enable_show_all',
+                                     $agp_enable_show_all);
+   }
    $agp_groups = array();
    if ($agp_grouping)
       $agp_groups = getPref($data_dir, $username,
@@ -862,6 +929,18 @@ function filter_address_book_listing_do(&$args)
          if (!empty($abook_entry['special_message'])) continue;
 
 
+         // if showing all for this backend, pretend that it is paginated already
+         //
+         if (!empty($agp_pagination[$abook_entry['backend']]['page_number'])
+          && $agp_pagination[$abook_entry['backend']]['page_number'] == 'all')
+         {
+            if ($agp_enable_show_all)
+               $abook_entry['paginated'] = TRUE;
+            else
+               $agp_pagination[$abook_entry['backend']]['page_number'] = 1;
+         }
+
+
          if (empty($agp_pagination[$abook_entry['backend']]['total_entries']))
          {
 
@@ -1054,9 +1133,14 @@ function agp_process_args($addresses, $backend=0)
    foreach ($agp_abook_backends as $backend)
    {
 
-      if (!sqGetGlobalVar('abook_groups_' . $backend, $agp_group_selections[$backend], SQ_FORM))
+      // note that "-- None --" is a special flag to turn off grouping
+      //
+      sq_change_text_domain('abook_group_pagination');
+      if (!sqGetGlobalVar('abook_groups_' . $backend, $agp_group_selections[$backend], SQ_FORM)
+       || $agp_group_selections[$backend][0] == _("-- None --"))
 //TODO:FIXME: in 1.5.2, this prevents keeping current group filters when changing backends, since it's one backend at a time... in this if(), we could attempt to search for other SQ_FORM args abook_groups_X with a different number to see if there is one(?) and use it(?)
          $agp_group_selections[$backend] = array();
+      sq_change_text_domain('squirrelmail');
 
       // turn filtering on for all backends unless their
       // group list is empty (don't need to use the page
